module fft_data_mod
  use fftwflags
  use fft_types
!  use status_mod
#ifndef USE_FFTW
  use fftw_sep_fake_mod
#endif
  integer, parameter, private :: dbl=selected_real_kind(10,50)
  complex,  allocatable :: tempc2(:,:,:),tempc1(:,:),tempcr(:,:)
  real,  allocatable :: temprc(:,:)
 
#ifdef SEP_OMP
  integer, external :: omp_get_thread_num
#endif

contains

  ! ----------------------------------------------------------------------
  function  init_fft_1d(n1,nth,fast_init) result (fft_id)
    type (fft_type) fft_id
    integer n1,stat,flag,nth,ith
    logical, optional :: fast_init


    if(present(fast_init)) then
       if(fast_init) then
          flag=ior(ior(fFTW_ESTIMATE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
       else
          flag=ior(ior(fFTW_MEASURE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
       end if
    else
       flag=ior(ior(fFTW_ESTIMATE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
    end if
    fft_id%scale=sqrt(1./(n1))
     allocate(fft_id%plan_back(nth))
     allocate(fft_id%plan_forw(nth))
     allocate(tempc1(n1,nth))

!    call fftw_f77_create_plan(fft_id%plan_forw,n1,fFTW_forward ,flag)
!    call fftw_f77_create_plan(fft_id%plan_back,n1,fFTW_BACKWARD,flag)
   do ith=1,nth
    call sfftw_plan_dft(fft_id%plan_back(ith),1,(/n1/),tempc1(1,ith),&   
      tempc1(1,ith),fFTW_BACKWARD,flag)
    call sfftw_plan_dft(fft_id%plan_forw(ith),1,(/n1/),tempc1(1,ith),&
       tempc1(1,ith),fFTW_FORWARD ,flag)
  end do
    if((fft_id%plan_forw(1) == 0 ).or.(fft_id%plan_back(1) == 0)) then
       fft_id%ierr=1
    else
       fft_id%ierr=0
    end if
    fft_id%ndim=1

  end function init_fft_1d
  ! ----------------------------------------------------------------------
  function init_rfft_1d(n1,nth,create) result (fft_id)
    type(fft_type)     :: fft_id
    integer, intent(in):: n1,nth
    integer            :: stat, flag,ith
    logical            :: create

!    flag=ior(ior(fFTW_ESTIMATE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
    flag=FFTW_PATIENT
    fft_id%scale=sqrt(1./(n1))
    if(create) then
      allocate(temprc(n1,nth),tempcr(n1/2+1,nth))
    end if
!    call rfftw_f77_create_plan(fft_id%plan_forw,n1,fFTW_REAL_TO_COMPLEX,flag)
!    call rfftw_f77_create_plan(fft_id%plan_back,n1,fFTW_COMPLEX_TO_REAL,flag)
    allocate(fft_id%plan_forw(nth))
    allocate(fft_id%plan_back(nth))
do ith=1,nth
    call sfftw_plan_dft_r2c(fft_id%plan_forw(ith),1,(/n1/),temprc(1,ith),tempcr(1,ith),&
     flag)
    call sfftw_plan_dft_c2r(fft_id%plan_back(ith),1,(/n1/),tempcr(1,ith),temprc(1,ith),&
     flag)
end do
    if((fft_id%plan_forw(1) == 0 ).or.(fft_id%plan_back(1) == 0)) then
       fft_id%ierr=1
    else
       fft_id%ierr=0
    end if
    fft_id%ndim=1
  end function init_rfft_1d
  ! ----------------------------------------------------------------------
  function  init_fft_2d(n1,n2,nth,fast_init) result (fft_id)
    type (fft_type) fft_id
    integer n1,n2,stat,flag,nth,ith
    logical, optional :: fast_init

    if(present(fast_init)) then
       if(fast_init) then
          flag=ior(ior(fFTW_ESTIMATE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
       else
          flag=ior(ior(fFTW_MEASURE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
       end if
    else
       flag=ior(ior(fFTW_ESTIMATE,fFTW_USE_WISDOM),fFTW_THREADSAFE)
    end if

   allocate(fft_id%plan_forw(nth))
   allocate(fft_id%plan_back(nth))
    fft_id%scale=sqrt(1./(n1*n2))
    allocate(tempc2(n1,n2,nth))
!    call fftw2d_f77_create_plan(fft_id%plan_forw,n1,n2,fFTW_FORWARD ,flag)
!    call fftw2d_f77_create_plan(fft_id%plan_back,n1,n2,fFTW_BACKWARD,flag)
do ith=1,nth
    call  sfftw_plan_dft(fft_id%plan_forw(ith),2,(/n1,n2/),&
      tempc2(1,1,ith),tempc2(1,1,ith),fFTW_FORWARD ,flag)
    call sfftw_plan_dft(fft_id%plan_back(ith),2,(/n1,n2/),&
      tempc2(1,1,ith),tempc2(1,1,ith),fFTW_BACKWARD,flag)
end do



    if((fft_id%plan_forw(1) ==0 ).or.(fft_id%plan_back(1) == 0)) then
       fft_id%ierr=1
    else
       fft_id%ierr=0
    end if

    fft_id%ndim=2
  end function init_fft_2d

  !----------------------------------------------------------------------
  subroutine fft_1d_data(fft_id,sign,n1,data_vec,ith)
    type (fft_type) fft_id
    integer :: sign, n1,ith
    complex, dimension (:) :: data_vec
 
    tempc1(1:n1,ith)=data_vec*fft_id%scale
    if(sign == fFTW_FORWARD) then
       call sfftw_execute(fft_id%plan_forw(ith))
    else
       call sfftw_execute(fft_id%plan_back(ith))
    end if
    return	
    data_vec(1:n1)=tempc1(1:n1,ith)
  end subroutine fft_1d_data
  !----------------------------------------------------------------------
  subroutine rfft_1d_data(fft_id,sign,n,real_vec,cmpx_vec,ith,verb)
  ! No side effects. cmpx_vec has both zero freq and Nyq
 
    type(fft_type),            intent(in   ) :: fft_id
    integer,                   intent(in   ) :: sign, n,ith
    real,    dimension(n    )                :: real_vec
    complex, dimension(n/2+1), intent(inout) :: cmpx_vec
    integer                                  :: i
    logical :: verb

    if(mod(n,2)==1) then
       write(0,*) "Error! rfft_1d_data needs a even n. Now, n =",n
    end if
    
    if(sign == fFTW_REAL_TO_COMPLEX) then
       temprc(1:n,ith)=real_vec(1:n)
       call sfftw_execute(fft_id%plan_forw(ith))
       cmpx_vec=conjg(tempcr(:,ith))*fft_id%scale
!       do i=1,size(cmpx_vec)
!      end do
    else
       tempcr(:,ith)=conjg(cmpx_vec)
       call sfftw_execute(fft_id%plan_back(ith))
       real_vec(1:n)=temprc(1:n,ith)*fft_id%scale
    end if
    return	
  end subroutine rfft_1d_data 
  !----------------------------------------------------------------------
  subroutine fft_2d_data(fft_id,sign,n1,n2,data_slice,ith)
    type (fft_type) fft_id
    integer :: n1,n2,ith,i1,i2
    integer :: sign
    complex, dimension (:,:) :: data_slice

    tempc2(:,:,ith)=data_slice*fft_id%scale

    if(sign == fFTW_FORWARD) then
       call sfftw_execute(fft_id%plan_back(ith))
    else
       call sfftw_execute(fft_id%plan_forw(ith))
    end if
    
    data_slice=(tempc2(:,:,ith))
    return	
  end subroutine fft_2d_data
  !----------------------------------------------------------------------
  subroutine clean_fft(fft_id) 
    type (fft_type) fft_id
    integer ierr,i

 do i=1,size(fft_id%plan_forw) 
    select case (fft_id%ndim)
    case (1)
       call sfftw_destroy_plan(fft_id%plan_forw(i))
       call sfftw_destroy_plan(fft_id%plan_back(i))
    case (2)
       call sfftw_destroy_plan(fft_id%plan_forw(i))
       call sfftw_destroy_plan(fft_id%plan_back(i))
    case (3)
       call sfftw_destroy_plan(fft_id%plan_forw(i))
       call sfftw_destroy_plan(fft_id%plan_back(i))
    case default
       write(0,*)'fft_id%ndim=',fft_id%ndim
       call seperr('Not supported fft_id%ndim')
    end select
 end do
       deallocate(fft_id%plan_forw)
       deallocate(fft_id%plan_back)
    return	
  end subroutine clean_fft
!-----------------------------------------------------
  subroutine clean_rfft(fft_id) 
    type (fft_type) fft_id
    integer ierr,ith

    select case (fft_id%ndim)
    case (1)
!       call rfftw_f77_destroy_plan(fft_id%plan_forw)
!       call rfftw_f77_destroy_plan(fft_id%plan_back)
      do ith=1,size(fft_id%plan_forw)
         call sfftw_destroy_plan(fft_id%plan_forw(ith))
         call sfftw_destroy_plan(fft_id%plan_back(ith))
      end do
       deallocate(fft_id%plan_forw)
       deallocate(fft_id%plan_back)
    case default
       write(0,*)'fft_id%ndim=',fft_id%ndim
       call seperr('Not supported fft_id%ndim')
    end select
    return	
  end subroutine clean_rfft
end module fft_data_mod

