!
! This is Sergey Fomel's dip finder
!
!  and Jon Claerbout Puck
!
!
!
module find_xz_dip_resid_mod
  use grad3_dip_mod
  use allpass_3d
  use allder_3d
  use divide
  implicit none
  contains
  
  logical function find_sergey_slow(n,field,resid,niter,nl_iter,eps,dip,verb&
    &)
    real,intent(in) :: field(:), resid(:)
    logical, intent(in) :: verb
    real :: eps,mean
    real,pointer ::  dip(:)
    integer, intent(in) :: niter,nl_iter
    integer :: n(3),ierr,n123,ndim,iter
    real, allocatable  :: u1(:),u2(:),dp(:)
    find_sergey_slow=.false.
    n123=product(n)
    allocate(u1(n123),u2(n123),dp(n123),stat=ierr)
    if (ierr.ne.0) then
      write(0,*) "trouble allocating temp arrays"
      return
    end if
    call grad3_init(n)
    call  allpass_3d_init(n(1),n(2),n(3),dip)
    call  allder_3d_init(n(1),n(2),n(3),dip)
    ndim=0
    if (n(1) >1) then
      ndim= ndim+1
    end if
    if (n(2) >1) then
      ndim= ndim+1
    end if
    if (n(3) >1) then
      ndim= ndim+1
    end if
    do iter=1,nl_iter 
      if (verb) then
        write(0,*) "working on non-linear iteration",iter
      end if
      ierr=allpass_3d_lop(.false.,.false.,field,u2)
      ierr=allder_3d_lop(.false.,.false.,field,u1)
      mean=sqrt(sum(u1*u1))
      u1=u1/mean
      u2=u2/mean
      call divide_apply_cg(.false.,n123,u2,u1,dp,eps,niter,grad3_op,ndim&
        &*n123)
      dip=dip+dp
    end do 
    ierr=allpass_3d_lop(.false.,.false.,field,resid)
    deallocate(dp,u1,u2)
    call allpass_3d_close()
    call allder_3d_close()
    call grad3_close()
    find_sergey_slow=.true.
  end function 
  logical function find_sergey_fast(n,field,resid,niter,eps1,eps2,dip,verb)
    logical, intent(in) :: verb
    real,pointer :: field(:), resid(:)
    real :: eps1,eps2,mean
    real,pointer ::  dip(:)
    integer, intent(in) :: niter
    integer :: n(3),ierr,n123,ndim,iter
    real, allocatable  :: u1(:),u2(:),dp(:)
    find_sergey_fast=.false.
    n123=product(n)
    allocate(u1(n123),u2(n123),dp(n123),stat=ierr)
    if (ierr.ne.0) then
      write(0,*) "trouble allocating temp arrays"
      return
    end if
    call  allpass_3d_init(n(1),n(2),n(3),dip)
    call  allder_3d_init(n(1),n(2),n(3),dip)
    dip=0.
    do iter=1,niter 
      if (verb) then
        write(0,*) "working on non-linear iteration",iter
      end if
      ierr=allpass_3d_lop(.false.,.false.,field,u2)
      ierr=allder_3d_lop(.false.,.false.,field,u1)
      mean=sqrt(sum(u1*u1))
      u1=u1/mean
      u2=u2/mean
      call divide_apply_tri(n,u2,u1,dp,eps1,eps2)
      dip=dip+dp
    end do 
    ierr=allpass_3d_lop(.false.,.false.,field,resid)
    deallocate(dp,u1,u2)
    call allpass_3d_close()
    call allder_3d_close()
    find_sergey_fast=.true.
  end function 
end module find_xz_dip_resid_mod
